/* ====================================================================
   Licensed to the Apache Software Foundation (ASF) under one or more
   contributor license agreements.  See the NOTICE file distributed with
   this work for additional information regarding copyright ownership.
   The ASF licenses this file to You under the Apache License, Version 2.0
   (the "License"); you may not use this file except in compliance with
   the License.  You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */

package org.apache.poi.ss.formula.functions;

import org.apache.poi.ss.formula.eval.*;

/**
 * Implementation for Excel DEC2OCT() function.<p/>
 * <p/>
 * <b>Syntax</b>:<br/> <b>DEC2OCT  </b>(<b>number</b>,<b>places</b> )<br/>
 * <p/>
 * Converts a decimal number to octal.
 * <p/>
 * The decimal integer you want to convert. If number is negative, places is ignored
 * and this function returns a 10-character (30-bit) octal number in which the most
 * significant bit is the sign bit. The remaining 29 bits are magnitude bits.
 * Negative numbers are represented using two's-complement notation.
 * <p/>
 * <ul>
 * <li>If number < --536,870,912 or if number > 536,870,911, this function returns the #NUM! error value.</li>
 * <li>If number is nonnumeric, this function returns the #VALUE! error value.</li>
 * </ul>
 * <p/>
 * <h2>places</h2>
 * <p/>
 * The number of characters to use. The places argument is useful for padding the
 * return value with leading 0s (zeros).
 * <p/>
 * <ul>
 * <li>If this function requires more than places characters, it returns the #NUM! error value.</li>
 * <li>If places is nonnumeric, this function returns the #VALUE! error value.</li>
 * <li>If places is negative, this function returns the #NUM! error value.</li>
 * <li>If places is not an integer, it is truncated.</li>
 * </ul>
 *
 * @author cedric dot walter @ gmail dot com
 */
public class Dec2Oct extends Var1or2ArgFunction {

    private final static long MIN_VALUE = new Long("-536870912").longValue();
    private final static long MAX_VALUE = new Long("536870911").longValue();
    private final static int DEFAULT_PLACES_VALUE = 10;

    @Override
    public ValueEval evaluate(int srcRowIndex, int srcColumnIndex, ValueEval number, ValueEval places) {
        ValueEval veText1;
        try {
            veText1 = OperandResolver.getSingleValue(number, srcRowIndex, srcColumnIndex);
        } catch (EvaluationException e) {
            return e.getErrorEval();
        }
        String strText1 = OperandResolver.coerceValueToString(veText1);
        Double number1 = OperandResolver.parseDouble(strText1);

        //If this number argument is non numeric, this function returns the #VALUE! error value.
        if (number1 == null) {
            return ErrorEval.VALUE_INVALID;
        }

        if (number1.longValue() < MIN_VALUE || number1.longValue() > MAX_VALUE) {
            return ErrorEval.NUM_ERROR;
        }

        int placesNumber = 0;
        if (number1 < 0) {
            placesNumber = DEFAULT_PLACES_VALUE;
        } else {
            ValueEval placesValueEval;
            try {
                placesValueEval = OperandResolver.getSingleValue(places, srcRowIndex, srcColumnIndex);
            } catch (EvaluationException e) {
                return e.getErrorEval();
            }
            String placesStr = OperandResolver.coerceValueToString(placesValueEval);
            Double placesNumberDouble = OperandResolver.parseDouble(placesStr);

            //non numeric value
            if (placesNumberDouble == null) {
                return ErrorEval.VALUE_INVALID;
            }

            //If this argument contains a decimal value, this function ignores the numbers to the right side of the decimal point.
            placesNumber = placesNumberDouble.intValue();

            if (placesNumber < 0) {
                return ErrorEval.NUM_ERROR;
            }
        }

        String octal = "";
        if (placesNumber != 0) {
            octal = String.format("%0" + placesNumber + "o", number1.intValue());
        } else {
            octal = Integer.toOctalString(number1.intValue());
        }

        //function returns a 10-character (30-bit) octal number in which the most significant bit is the sign bit. The remaining 29 bits are magnitude bits
        if (octal.length() > 10) {
            octal = octal.substring(1);
        }

        if (placesNumber < octal.length()) {
            return ErrorEval.NUM_ERROR;
        }

        return new StringEval(octal);
    }

    @Override
    public ValueEval evaluate(int srcRowIndex, int srcColumnIndex, ValueEval arg0) {
        return this.evaluate(srcRowIndex, srcColumnIndex, arg0, new StringEval(String.valueOf(DEFAULT_PLACES_VALUE)));
    }
}
